﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;

/* --- © 2022 JaxZoa <JaxZoa@gmail.com> --- */

namespace JaxPlugins
{

    public class Step
    {
        protected StepCycle cycle;

        protected float duration;
        protected float delay;
        protected bool right;

        protected float startTime;

        protected Vector3 startPos;
        protected Vector3 tarPos;

        protected Quaternion startRot;
        protected Quaternion tarRot;

        protected bool isActive = false;

        protected float effectiveDuration = 1;

        protected float stepHeightDelta = 0;

        public Step(StepCycle _cycle, float _duration, float _delay, bool _right)
        {
            cycle = _cycle;

            duration = _duration;
            delay = _delay;
            right = _right;
        }

        public void Init(Vector3 _startPos, Quaternion _startRot)
        {
            tarRot = _startRot;
            tarPos = _startPos;
        }

        public void StartStep(Vector3 _tarPos, Quaternion _tarRot)
        {
            isActive = true;

            stepHeightDelta =  _tarPos.y - tarPos.y;

            startPos = tarPos;
            startRot = tarRot;

            tarPos = _tarPos;
            tarRot = _tarRot;

            startTime = Time.time;

            if (cycle.StepInPlace)
                effectiveDuration = duration;
            else
                effectiveDuration = Mathf.Lerp(duration * 0.5f, duration, Mathf.Clamp01(StepLength));
        }

        public void ResetStep()
        {
            isActive = false;
        }

        public float StepTime
        {
            get { return Mathf.Clamp01((Time.time - startTime) / effectiveDuration); }
        }

        public float StepLength
        {
            get { return (startPos - tarPos).magnitude; }
        }

        public float GetStepStrength(bool applyFilters = true)
        {
            float _str = Mathf.Pow((4 * StepTime) * (1 - StepTime), cycle.Sharpness);

            if (applyFilters)
            {
                if (cycle.StepInPlace)
                    _str *= cycle.StepInPlaceAmplitude;
                else
                    _str = Mathf.Lerp(0, _str, Mathf.Clamp01(StepLength));
            }

            return _str;
        }

        public Quaternion TargetRot
        {
            get { return Quaternion.Lerp(startRot, tarRot, StepTime); }
        }

        public Vector3 TargetPos
        {
            get { return Vector3.Lerp(startPos, tarPos, StepTime); }
        }

        public bool IsActive
        {
            get { return isActive; }
        }

        public bool PlayedThisCycle
        {
            get { return Mathf.Clamp01((Time.time - startTime) / (effectiveDuration + delay)) >= 1; }
        }

        public bool IsIdle
        {
            get { return tarPos == startPos && tarRot == startRot; }
        }

        public float StepHeightDelta
        {
            get { return stepHeightDelta; }
        }

        public float Duration
        {
            get { return duration; }

            set { duration = value; }
        }

        public float Delay
        {
            get { return delay; }

            set { delay = value; }
        }

        public bool Right
        {
            get { return right; }
        }
    }

}