﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;

/* --- © 2022 JaxZoa <JaxZoa@gmail.com> --- */

namespace JaxPlugins
{

    public class StepCycle
    {
        protected CLS manager;

        protected List<Step> steps;

        protected float stepSpacingIdle;
        protected float stepSpacingMoving;
        protected float stepSpacingKneelingIdle;
        protected float stepSpacingKneelingMoving;

        protected float groundOffset;
        protected float groundOffsetKneeling;

        protected bool stepInPlace;
        protected float stepInPlaceAmplitude;

        protected float sharpness;

        protected bool isIdle;

        public StateChange idleStateDelegate;
        public delegate void StateChange(bool _newState);

        protected int activeStep = 0;

        public StepCycle(CLS _manager)
        {
            manager = _manager;
        }

        public void SetSteps(List<Step> _steps)
        {
            steps = _steps;
        }

        public void Init()
        {
            foreach(Step _step in steps)
            {
                _step.Init(ComputeStepTargetPos(_step), manager.TargetTransform.rotation);
            }
        }

        public void ComputeSteps()
        {
            if(!steps[activeStep].IsActive)
            {
                bool idleState = false;
                
                steps[activeStep].StartStep(ComputeStepTargetPos(steps[activeStep]), manager.TargetTransform.rotation);

                foreach (Step _step in steps)
                {
                    if (_step.IsIdle)
                        idleState = true;
                }

                if(idleState != isIdle)
                {
                    isIdle = idleState;

                    if(isIdle)
                        idleStateDelegate.Invoke(isIdle);
                }
            }
            else
            {
                if (steps[activeStep].PlayedThisCycle)
                    activeStep++;

                if (activeStep >= steps.Count)
                {
                    foreach(Step _step in steps)
                    {
                        _step.ResetStep();
                    }

                    activeStep = 0;
                }
            }
        }

        RaycastHit[] hitInfos = new RaycastHit[100];
        int hitCount = 0;
        float stepSpacing = 0;
        Vector3 stepTargetPos;
        Vector3 stepRayCastPos;

        protected Vector3 ComputeStepTargetPos(Step _step)
        {
            stepTargetPos = manager.TargetTransform.position;

            float _stepHeight = -10000;

            if (manager.IsStanding)
            {
                stepSpacing = Mathf.Lerp(stepSpacingIdle, stepSpacingMoving, manager.SpeedFactor);
            }
            else
            {
                stepSpacing = Mathf.Lerp(stepSpacingKneelingIdle, stepSpacingKneelingMoving , manager.SpeedFactor);
            }

            if (!_step.Right)
                stepSpacing = -stepSpacing;

            stepTargetPos += manager.TargetTransform.right * stepSpacing;

            stepRayCastPos = stepTargetPos;

            hitCount = Physics.SphereCastNonAlloc(stepRayCastPos + Vector3.up * 0.5f, 0.05f, -Vector3.up, hitInfos, 1f);

            if (hitCount > 0)
            {
                for(int i = 0; i < hitCount; i++)
                {
                    if (hitInfos[i].collider == null)
                        continue;

                    if (!manager.GroundColliders.Contains(hitInfos[i].collider))
                        continue;


                    if (hitInfos[i].point.y > _stepHeight)
                    {
                        _stepHeight = hitInfos[i].point.y;
                    }
                }
            }

            if (_stepHeight == -10000)
                _stepHeight = stepTargetPos.y;

            _stepHeight += manager.IsStanding ? groundOffset : groundOffsetKneeling;

            stepTargetPos.y = _stepHeight;

            return stepTargetPos;
        }

        public Step GetStep(int _stepIndex)
        {
            if (_stepIndex < 0 || _stepIndex >= steps.Count)
                return null;

            return steps[_stepIndex];
        }

        public bool IsIdle
        {
            get { return isIdle; }
        }

        public float StepSpacingIdle
        {
            get { return stepSpacingIdle; }

            set { stepSpacingIdle = value; }
        }

        public float StepSpacingMoving
        {
            get { return stepSpacingMoving; }

            set { stepSpacingMoving = value; }
        }

        public float StepSpacingKneelingIdle
        {
            get { return stepSpacingKneelingIdle; }

            set { stepSpacingKneelingIdle = value; }
        }

        public float StepSpacingKneelingMoving
        {
            get { return stepSpacingKneelingMoving; }

            set { stepSpacingKneelingMoving = value; }
        }

        public float Sharpness
        {
            get { return sharpness; }

            set { sharpness = value; }
        }

        public float GroundOffset
        {
            get { return groundOffset; }

            set { groundOffset = value; }
        }

        public float GroundOffsetKneeling
        {
            get { return groundOffsetKneeling; }

            set { groundOffsetKneeling = value; }
        }

        public float StepInPlaceAmplitude
        {
            get { return stepInPlaceAmplitude; }

            set { stepInPlaceAmplitude = value; }
        }

        public bool StepInPlace
        {
            get { return stepInPlace; }

            set { stepInPlace = value; }
        }
    }
}